﻿using EasyCaching.Core;
using Hangfire;
using Hangfire.Storage;
using IRaCIS.Core.Application.Helper;
using IRaCIS.Core.Application.Service;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Infrastructure;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using SharpCompress.Common;
using ZiggyCreatures.Caching.Fusion;

namespace IRaCIS.Application.Services.BackGroundJob
{

    public interface IIRaCISHangfireJob
    {

        Task MemoryCacheTrialStatusAsync();

        Task InitHangfireJobTaskAsync();

    }
    public class IRaCISCHangfireJob(IRepository<Trial> _trialRepository,
         IEasyCachingProvider _provider,
         ILogger<IRaCISCHangfireJob> _logger,
         IRepository<SystemAnonymization> _systemAnonymizationRepository,
         IRepository<Internationalization> _internationalizationRepository,
         IRepository<TrialEmailNoticeConfig> _trialEmailNoticeConfigRepository
        ) : IIRaCISHangfireJob
    {
        public static string JsonFileFolder = Path.Combine(AppContext.BaseDirectory, StaticData.Folder.Resources);


        public async Task InitHangfireJobTaskAsync()
        {
            _logger.LogInformation("项目启动 hangfire 任务初始化 执行开始~");


            //项目状态 立即加载到缓存中
            await MemoryCacheTrialStatusAsync();

            //await MemoryCacheAnonymizeData();


            //创建项目缓存 定时任务
            HangfireJobHelper.AddOrUpdateInitCronJob<IIRaCISHangfireJob>("RecurringJob_Cache_TrialState", t => t.MemoryCacheTrialStatusAsync(), Cron.Daily());

            //初始化

            await InternationalizationHelper.InitInternationlizationDataAndWatchJsonFileAsync(_internationalizationRepository);

            //创建邮件定时任务
            await InitSysAndTrialCronJobAsync();


            _logger.LogInformation("项目启动 hangfire 任务初始化 执行结束");
        }







        public async Task InitSysAndTrialCronJobAsync()
        {
            //var deleteJobIdList = await _trialEmailNoticeConfigRepository.Where(t => t.Trial.TrialStatusStr != StaticData.TrialState.TrialOngoing && t.EmailCron != string.Empty && t.IsAutoSend)
            //   .Select(t => t.TrialId + "_" + t.Id)
            //    .ToListAsync();

            //foreach (var jobId in deleteJobIdList)
            //{
            //    HangfireJobHelper.RemoveCronJob(jobId);
            //}


            var taskInfoList = await _trialEmailNoticeConfigRepository.Where(t => t.Trial.TrialStatusStr == StaticData.TrialState.TrialOngoing && t.EmailCron != string.Empty && t.IsAutoSend)
                  .Select(t => new { t.Id, t.Code, t.EmailCron, t.BusinessScenarioEnum, t.TrialId })
                  .ToListAsync();

            foreach (var task in taskInfoList)
            {
                //利用主键作为任务Id
                var jobId = $"{task.TrialId}_{task.Id}";

                HangfireJobHelper.AddOrUpdateTrialCronJob(jobId, task.TrialId, task.BusinessScenarioEnum, task.EmailCron);
            }

            var addOrUpdateJobIdList = taskInfoList.Select(t => $"{t.TrialId}_{t.Id}").ToList();

            var list = JobStorage.Current.GetConnection().GetRecurringJobs().ToList();

            //项目定时任务都在default 队列
            //var dbJobIdList = JobStorage.Current.GetConnection().GetRecurringJobs().Where(t => t.Queue == "default").Select(t => t.Id).ToList();

            //var deleteList=  dbJobIdList.Except(addOrUpdateJobIdList).ToList();

            //  foreach (var jobId in deleteList)
            //  {
            //      HangfireJobHelper.RemoveCronJob(jobId);
            //  }


        }


        #region 废弃 前端上传的时候获取


        /// <summary>
        /// 缓存项目状态--之前是启动的时候就获取所有的项目进行缓存，加上定时任务刷新，现在的话，改为是按照需要进行缓存请求数据库
        /// </summary>
        /// <returns></returns>
        public async Task MemoryCacheTrialStatusAsync()
        {

            var list = await _trialRepository.Select(t => new { TrialId = t.Id, TrialStatusStr = t.TrialStatusStr })
                 .ToListAsync();

            list.ForEach(t => _provider.Set(t.TrialId.ToString(), t.TrialStatusStr, TimeSpan.FromDays(7)));

        }
        public async Task MemoryCacheAnonymizeData()
        {
            var systemAnonymizationList = await _systemAnonymizationRepository.Where(t => t.IsEnable).ToListAsync();

            _provider.Set(StaticData.Anonymize.Anonymize_AddFixedFiled, systemAnonymizationList.Where(t => t.IsAdd && t.IsFixed).ToList(), TimeSpan.FromDays(7));
            _provider.Set(StaticData.Anonymize.Anonymize_AddIRCInfoFiled, systemAnonymizationList.Where(t => t.IsAdd && t.IsFixed == false).ToList(), TimeSpan.FromDays(7));
            _provider.Set(StaticData.Anonymize.Anonymize_FixedField, systemAnonymizationList.Where(t => t.IsAdd == false && t.IsFixed).ToList(), TimeSpan.FromDays(7));
            _provider.Set(StaticData.Anonymize.Anonymize_IRCInfoField, systemAnonymizationList.Where(t => t.IsAdd == false && t.IsFixed == false).ToList(), TimeSpan.FromDays(7));
        }


        #endregion
    }


}
